settings = require "settings"
managers = require "managers"
actors = require "actors"

local map -- stores tiledata
local mapX, mapY -- view x,y in tiles. can be a fractional value like 3.25.
local oldMapX, oldMapY

local tilesDisplayWidth, tilesDisplayHeight -- number of tiles to show
local scale

local tilesetSprite

function setupMap()
    local tileQuads = {actors.Floor, actors.Wall}
    map = {}
    for x = 1, settings.world.tilesWidth do
        map[x] = {}
        for y = 1, settings.world.tilesHeight do
            local i = 1
            if math.random() > 0.8 then
                i = 2
            end
            map[x][y] = tileQuads[i](x, y)
        end
    end
end

function setupMapView(viewScale)
    mapX = 1
    mapY = 1
    oldMapX = 1
    oldMapY = 1

    local size = settings.world.tileWorldSize * viewScale  -- The size of the tiles in the view.
    tilesDisplayWidth = math.ceil(settings.window.width / size) + 1
    tilesDisplayHeight = math.ceil(settings.window.height / size) + 1
    
    scale = settings.world.tileWorldSize / settings.world.tilePixelSize
end

function setupTileset()
    tilesetBatch = love.graphics.newSpriteBatch(settings.world.tilesetImage, tilesDisplayWidth * tilesDisplayHeight)
    updateTilesetBatch()
end

function updateTilesetBatch()
    tilesetBatch:clear()

    -- init
    local startX, startY = 1, 1
    local endX, endY = tilesDisplayWidth, tilesDisplayHeight

    local endW = settings.world.tilesWidth - tilesDisplayWidth
    if mapX < 0 then
        startX = 1 + -mapX
    end
    if mapX > endW then
        endX = endX - (mapX - endW)
    end

    local endH = settings.world.tilesHeight - tilesDisplayHeight
    if mapY < 0 then
        startY = 1 + -mapY
    end
    if mapY > endH then
        endY = endY - (mapY - endH)
    end

    for x=startX, endX do
        for y=startY, endY do
            local quad = map[x+mapX][y+mapY].quad
            tilesetBatch:addq(quad,
                (x-1)*settings.world.tilePixelSize, (y-1)*settings.world.tilePixelSize)
        end
    end
end

-- central function for moving the map
moveMap = function(x, y)
    mapX = x
    mapY = y
    -- only update if we actually moved
    --if mapX ~= oldMapX or mapY ~= oldMapY then
        updateTilesetBatch()
    --end
    oldMapX = mapX
    oldMapY = mapY
end;


return {
    load = function(viewScale)
        setupMap()
        setupMapView(viewScale)
        setupTileset()
    end;

    setupMapView = setupMapView,
    setupTileset = setupTileset,

    draw = function(camera)
        local v0x, v0y = camera:worldCoords(0, 0)
        local d0x = math.floor(v0x / settings.world.tileWorldSize)
        local d0y= math.floor(v0y / settings.world.tileWorldSize)
        moveMap(d0x, d0y)
        
        love.graphics.draw(tilesetBatch,
            d0x * settings.world.tileWorldSize, d0y * settings.world.tileWorldSize,
            0, scale, scale)
    end;

    checkRange = function(pos, paddingX, paddingY)
        if paddingY == nil then
            paddingY = paddingX
        end
        if pos.x < paddingX then
            pos.x = paddingX
        elseif pos.x > (settings.world.width - paddingX) then
            pos.x = settings.world.width - paddingX
        end
        if pos.y < paddingY then
            pos.y = paddingY
        elseif pos.y > (settings.world.height - paddingY) then
            pos.y = settings.world.height - paddingY
        end
        return pos
    end;
}