local layers = {}
local toAddes = {}
local toRemoves = {}
local counter = 0

function add(obj, layer)
    local objects = layers[layer]
    if objects == nil then
        objects = {}
        layers[layer] = objects
    end
    obj._layer = layer
    objects[obj] = true
end

function remove(obj)
    obj.physics.fixture:destroy()
    obj.physics.body:destroy()
    layers[obj._layer][obj] = nil
end

function beginContact(a, b, coll)
    
end

function endContact(a, b, coll)
    
end

function preSolve(a, b, coll)
    local actorA = a:getUserData()
    local actorB = b:getUserData()

    if actorA.type > actorB.type then
        actorA, actorB = actorB, actorA
    end

    if actorA.type == 'Bullet' and (actorB.type == 'Minion' or actorB.type == 'Wall') then
        actorB:hit(actorA.damage)
        if actorB:isDead() then
            actorB:kill()
        end
        actorA:remove()
    end
end

function postSolve(a, b, coll)
    
end


love.physics.setMeter(1)
local world = love.physics.newWorld(0, 0, true)
world:setCallbacks(beginContact, endContact, preSolve, postSolve)

local tileSize = settings.world.tilePixelSize -- size of tiles in pixels
local tilesetImage = settings.world.tilesetImage
tilesetImage:setFilter("nearest", "nearest") -- this "linear filter" removes some artifacts if we were to scale the tiles

return {
    world = world,

    tiles = {
        newQuad = function(x, y)
            return love.graphics.newQuad(x * tileSize, y * tileSize, tileSize, tileSize,
                                         tilesetImage:getWidth(), tilesetImage:getHeight())
        end;
    },

    objects = {
        player = nil,

        add = function(obj, layer)
            toAddes[obj] = layer or 0
        end;

        remove = function(obj)
            toRemoves[obj] = true
        end;

        doChanges = function()
            for obj, _ in pairs(toRemoves) do
                remove(obj)
                counter = counter - 1
            end
            toRemoves = {}
            for obj, layer in pairs(toAddes) do
                add(obj, layer)
                counter = counter + 1
            end
            toAddes = {}
        end;

        count = function()
            return counter
        end;

        all = function(nameMethod, dt)
            for layer, objects in pairs(layers) do
                for obj, _ in pairs(objects) do
                    obj[nameMethod](obj, dt)
                end
            end
        end;
    }
}