settings = require "settings"
managers = require "managers"
ground = require "ground"
actors = require "actors"
Vector = require "hump.vector"
Camera = require "hump.camera"
Timer = require "hump.timer"

local player
local camera
local scaleTarget, scaleHandle, closerHandle
local scaleSpeed = 1.2
local scaleMethod = 'linear'

return {
    load = function()
        player = actors.Player(Vector(50.442, 49.745))
        managers.objects.player = player
        camera = Camera(player:getX(), player:getY())
        scaleTarget = 100
        ground.load(scaleTarget)
        scaleHandle = Timer.tween(.1, camera, {scale = scaleTarget}, scaleMethod)

        for i = 1, 100 do
            local pos = Vector(settings.world.width * math.random(), settings.world.height * math.random())
            actors.Minion(pos, 1)
        end
    end;

    mousepressed = function(x, y, button)
        if button == "wu" then
            scaleTarget = math.abs(camera.scale * scaleSpeed)
            Timer.cancel(scaleHandle)
            scaleHandle = Timer.tween(.1, camera, {scale = scaleTarget}, scaleMethod)
            if closerHandle ~= nil then
                Timer.cancel(closerHandle)
            end
            closerHandle = Timer.add(.1, function()
                ground.setupMapView(scaleTarget)
                ground.setupTileset()
            end)
        end
        if button == "wd" then
            scaleTarget = math.abs(camera.scale / scaleSpeed)
            Timer.cancel(scaleHandle)
            scaleHandle = Timer.tween(.1, camera, {scale = scaleTarget}, scaleMethod)
            if closerHandle ~= nil then
                Timer.cancel(closerHandle)
            end
            ground.setupMapView(scaleTarget)
            ground.setupTileset()
        end
    end;

    update = function (dt)
        if love.keyboard.isDown("escape")  then
            love.event.push("quit")
        end

        Timer.update(dt)

        managers.world:update(dt)

        if love.mouse.isDown("l") then
            local pos = Vector(camera:worldCoords(love.mouse.getPosition()))
            player:fire(pos)
        end

        managers.objects.all('update', dt)
        managers.objects.doChanges()
        local dx,dy = player:getX() - camera.x, player:getY() - camera.y
        camera:move(dx/2, dy/2)
    end;

    draw = function()
        camera:attach()
            ground.draw(camera)
            managers.objects.all('draw', dt)
        camera:detach()
        love.graphics.print("FPS: "..love.timer.getFPS(), 10, 20)
        love.graphics.print("Actors: "..managers.objects.count(), 10, 40)
        love.graphics.print("Pos: "..string.format("%g, %g", camera.x, camera.y), 10, 60)
    end;
}