Class = require "hump.class"
Vector = require "hump.vector"
settings = require "settings"


Weapon = Class{
    magazine,  -- quantity
    reloadTime,  -- seconds
    fireRate,  -- seconds per shot
    bulletSpeed,

    init = function(self)
        self.currentCharges = self.magazine
        self.reloadTicks = nil
        self.shotTicks = nil
    end;

    update = function(self, dt)
        if self:isReload() then
            self.reloadTicks = self.reloadTicks + dt
            if self.reloadTicks >= self.reloadTime then
                self.reloadTicks = nil
            end
        elseif self.shotTicks ~= nil then
            self.shotTicks = self.shotTicks + dt
            if self.shotTicks >= self.fireRate then
                self.shotTicks = nil
            end
        end
    end;

    isReload = function(self)
        return self.reloadTicks ~= nil
    end;

    doReload = function(self)
        self.currentCharges = self.magazine
        self.reloadTicks = 0.0
        self.shotTicks = nil
    end;

    canShot = function(self)
        return self.reloadTicks == nil and self.shotTicks == nil
    end;

    tryShot = function(self)
        if self.shotTicks == nil then
            self.shotTicks = 0.0
            self.currentCharges = self.currentCharges - 1
            return true
        end
    end;

    doShot = function(self)
        if self.reloadTicks == nil then
            if self.currentCharges > 0 then
                if self:tryShot() then
                    return true
                end
            else
                self:doReload()
            end
        end
        return false
    end;

    spawnShot =function(self, Bullet, pos, impulseDir)
        Bullet(pos, impulseDir * self.bulletSpeed)
        love.audio.stop(settings.world.bullet.song)
        love.audio.play(settings.world.bullet.song)
    end;
}


SPEED_K = 1000


return {
    Beretta_92 = Class{__includes = Weapon;
        reloadTime = 1.5,
        magazine = 15,
        fireRate = 1.0,
        bulletSpeed = 390.0 / SPEED_K,
    },


    HK_MP5 = Class{__includes = Weapon;
        reloadTime = 1.5,
        magazine = 30,
        fireRate = 60.0 / 800.0,  -- 800 shots per minute
        bulletSpeed = 400.0 / SPEED_K,
    },


    Jackhammer = Class{__includes = Weapon;
        reloadTime = 2.0,
        magazine = 30,
        fireRate = 60.0 / 240.0,  -- 240 shots per minute
        bulletSpeed = 400.0 / SPEED_K,

        spawnShot = function(self, Bullet, pos, impulseDir)
            local maxDev = math.pi / 8.0
            local projectiles = 7

            local stepDev = maxDev / projectiles
            local side
            if math.random(0,1) == 1 then 
                side = 1
            else
                side = -1
            end
            for i = 1, projectiles do
                local angel = stepDev * i * math.random()
                local newImpulse = impulseDir:rotate_inplace(angel * side)
                Bullet(pos, impulseDir * self.bulletSpeed)
                side = -side
            end

            love.audio.play(settings.world.bullet.song)
        end;
    },
}